using System;
using System.Net;
using System.Collections.Generic;
using System.IO;

namespace Subgurim.Controles.SSM
{
    /// <summary>
    /// Summary description for SiteMap
    /// </summary>
    public class SubgurimSiteMapBase
    {

        #region SiteMap properties
        private string _loc = string.Empty;
        /// <summary>
        /// Url de la pgina
        /// Url page
        /// </summary>
        public string loc
        {
            get { return _loc; }
            set { _loc = value.ToLower(); }
        }

        private DateTime _lastmod = DateTime.UtcNow;
        /// <summary>
        /// ltima vez que se modific
        /// Last time modified
        /// </summary>
        public DateTime lastmod
        {
            get { return _lastmod; }
            set { _lastmod = value; }
        }

        private freq _changefreq = freq.weekly;
        /// <summary>
        /// Frecuencia de actualizacin de la pgina
        /// Change page frequency
        /// </summary>
        public freq changefreq
        {
            get { return _changefreq; }
            set { _changefreq = value; }
        }

        private double _priority = 0.5;
        /// <summary>
        /// Prioridad. Va entre 0.0 y 1.0
        /// Priority. From 0.0 to 1.0
        /// </summary>
        public double priority
        {
            get
            {
                return Math.Round(_priority, 2);
            }
            set
            {
                _priority = _priority > 1.0 ? 1.0 : _priority < 0.0 ? 0.0 : _priority;
                _priority = value;
            }
        }

        #endregion

        #region Extra properties
        private string _hash = string.Empty;
        /// <summary>
        /// Hash del ltimo contenido de la pgina
        /// Hash from the last content of the page
        /// </summary>
        public string hash
        {
            get { return _hash; }
            set { _hash = value; }
        }

        const int maxLastModListItems = 5;

        private List<DateTime> _lastmodList = new List<DateTime>();
        /// <summary>
        /// Listado de las ltima modificaciones de la pgina.
        /// Nos servir para el changefreq y el priority.
        /// 
        /// List of the last page changes.
        /// We will use it for changefreq and priority
        /// </summary>
        public List<DateTime> lastmodList
        {
            get
            {
                if (_lastmodList.Count == 0)
                    _lastmodList.Add(lastmod);
                else if (_lastmodList.Count > maxLastModListItems)
                    _lastmodList.RemoveAt(0);

                return _lastmodList;
            }
            set
            {
                _lastmodList = value;
            }
        }

        private DateTime _lastAccess = DateTime.UtcNow;
        /// <summary>
        /// Determina cuando fue el ltimo acceso
        /// Determines the last access
        /// </summary>
        public DateTime lastAccess
        {
            get { return _lastAccess; }
            set { _lastAccess = value; }
        }

        #endregion

        public enum freq
        {
            always, hourly, daily, weekly, monthly, yearly, never
        }

        public SubgurimSiteMapBase()
        { }

        public SubgurimSiteMapBase(string loc)
        {
            this.loc = loc;
        }


        #region Methods

        #region Hash
        /// <summary>
        /// Recoge el Hash de la Web que define loc y lo aplica a nuestra propieda hash
        /// Take the Hash forn the site and sets it to our hash
        /// </summary>
        public void executeHash()
        {
            this.hash = computeHash(loc);
        }

        /// <summary>
        /// Create the hash
        /// </summary>
        /// <param name="loc"></param>
        /// <returns></returns>
        public static string computeHash(string loc)
        {
            string retorno = string.Empty;
            if (!string.IsNullOrEmpty(loc))
            {
                string delim = SubgurimSiteMapUtilities.getDelim(loc);

                Uri url = new Uri(loc + delim + SubgurimSiteMapUtilities.SubgurimSiteMapQuery + "=1");

                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
                try
                {
                    using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                    {
                        if (response.StatusCode == HttpStatusCode.OK)
                        {
                            StreamReader sr = new StreamReader(response.GetResponseStream());
                            retorno = SubgurimSiteMapUtilities.getHash(sr.ReadToEnd());
                        }
                    }
                }
                catch
                { }
            }

            return retorno;
        }

        #endregion

        /// <summary>
        /// Modifica las variables lastmod y lista de lastmode
        /// Modifies lastmod and lastmodlist variables
        /// </summary>
        public void updateLastMod(DateTime time)
        {
            this.lastmod = time;
            this.lastmodList.Add(time);
        }

        /// <summary>
        /// Computa si se debe cambiar changefreq y priority en base a lastmodlist
        /// Determines if changefreq and priority should be changed using lasmodlist
        /// </summary>
        public void computeVariables()
        {
            if (lastmodList.Count > 1)
            {
                int total = lastmodList.Count - 1;
                // last indica lo que consideramos como ltimas actualizaciones
                int last = maxLastModListItems - 1;
                last = total > last ? last : total;

                // Calculamos los ticks de media
                long ticks
                    = (long)((lastmodList[total].Ticks - lastmodList[total - last].Ticks) / (total + 1));

                double days = new TimeSpan(ticks).TotalDays;

                this.changefreq = computeChangefreq(days);
                this.priority = computePriority(days);
            }
        }

        /// <summary>
        /// En base a los das que ha tardado desde la ltima actualizacin, dar una frecuencia u otra
        /// Determines the frequency, based on the days since last modification
        /// </summary>
        /// <param name="days"></param>
        /// <returns></returns>
        private freq computeChangefreq(double days)
        {
            freq frequency = freq.weekly;
            if (days < 1.5)
            {
                frequency = freq.always;
            }
            else if (days < 3)
            {
                frequency = freq.hourly;
            }
            else if (days < 7)
            {
                frequency = freq.daily;
            }
            else if (days < 21)
            {
                frequency = freq.weekly;
            }
            else if (days < 300)
            {
                frequency = freq.monthly;
            }
            else if (days < 1000)
            {
                frequency = freq.yearly;
            }
            else
            {
                frequency = freq.never;
            }

            return frequency;
        }

        private double computePriority(double days)
        {
            return Math.Exp(days / -7);
        }
        
        #endregion
    }
}